using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Web;
using AutomobileSampleDataModel;
using WcfRestUtilities;

namespace CatchAllAutomobileService
{
    [ServiceBehavior(ConcurrencyMode=ConcurrencyMode.Single)]
    public class CatchAllAutomobileService : IAutomobileService
    {
        private static UriTemplateTable _table;
        public UriTemplateTable TemplateTable
        {
            get
            {
                if (_table == null)
                {
                    var request = WebOperationContext.Current.IncomingRequest;
                    _table = new UriTemplateTable(request.UriTemplateMatch.BaseUri, AutomobileServiceConstants.GetTemplates());
                }
                return _table;
            }
        }

        public Message Get()
        {
            var values = GetMatchedValues();

            return GetInternal(values);
        }

        private Message GetInternal(AutomobileMatchObject values)
        {
            // Get
            if (!string.IsNullOrEmpty(values.Engine))
            {
                var result = AutomobileRepository.Instance.GetAutomobile(values.YearValue, values.Make, values.Model, values.Engine, GetEngineLink);
                if (result == null)
                {
                    RestHelpers.SetStatusNotFound();
                    return RestHelpers.CreateEmptyMessage().WithHttpStatus();
                }
                return CreateMessage(result);
            }
            if (!string.IsNullOrEmpty(values.Model))
            {
                return CreateMessage(AutomobileRepository.Instance.GetEngines(values.YearValue, values.Make, values.Model, GetEngineLink));
            }
            if (!string.IsNullOrEmpty(values.Make))
            {
                return CreateMessage(AutomobileRepository.Instance.GetModels(values.YearValue, values.Make, GetModelLink));
            }
            if (values.YearValue > 0)
            {
                return CreateMessage(AutomobileRepository.Instance.GetMakes(values.YearValue, GetMakeLink));
            }
            return CreateMessage(AutomobileRepository.Instance.GetYears(GetYearLink));
        }

        public Message Post(Message data)
        {
            Automobile automobile = GetData<Automobile>(data);
            if (automobile == null)
            {
                RestHelpers.SetStatusBadRequest();
                return RestHelpers.CreateEmptyMessage().WithHttpStatus();
            }

            var result = AutomobileRepository.Instance.Insert(automobile);
            if (result == 0)
            {
                RestHelpers.SetStatusNotFound();
                return RestHelpers.CreateEmptyMessage().WithHttpStatus();
            }
            else
            {
                RestHelpers.SetStatusCreated();
                return GetInternal(AutomobileMatchObject.CopyFrom(automobile));
            }
        }

        public Message Put(Message data)
        {
            Automobile automobile = GetData<Automobile>(data); ;
            if (automobile == null)
            {
                RestHelpers.SetStatusBadRequest();
                return RestHelpers.CreateEmptyMessage().WithHttpStatus();
            }

            var values = GetMatchedValues();
            var result = AutomobileRepository.Instance.Update(values.YearValue, values.Make, values.Model, values.Engine, automobile);
            if (result == 0)
            {
                RestHelpers.SetStatusNotFound();
                return RestHelpers.CreateEmptyMessage().WithHttpStatus();
            }
            else
            {
                return GetInternal(AutomobileMatchObject.CopyFrom(automobile));
            }
        }

        public Message Delete()
        {
            var values = GetMatchedValues();
            var result = AutomobileRepository.Instance.Delete(values.YearValue, values.Make, values.Model, values.Engine);
            if (result)
            {
                RestHelpers.SetStatusOK();
                return RestHelpers.CreateEmptyMessage().WithHttpStatus();
            }
            else
            {
                RestHelpers.SetStatusNotFound();
                return RestHelpers.CreateEmptyMessage().WithHttpStatus();
            }
        }

        public AutomobileMatchObject GetMatchedValues()
        {
            return RestHelpers.GetMatchedValues<AutomobileMatchObject>(GetCurrentMatch());
        }

        public UriTemplateMatch GetCurrentMatch()
        {
            return RestHelpers.GetCurrentMatch(TemplateTable);
        }

        private T GetData<T>(Message data)
        {
            if (IsJsonRequest)
            {

                return data.GetJsonBody<T>();
            }
            else
            {
                return data.GetXmlBody<T>();
            }
        }

        private Message CreateMessage<T>(T result)
        {
            if (IsJsonRequest)
            {
                return RestHelpers.CreateJsonMessage(result).WithHttpStatus();
            }
            else
            {
                return RestHelpers.CreateXmlSerializedMessage(result).WithHttpStatus();
            }
        }

        #region Get Relative Links
        public bool IsJsonRequest
        {
            get
            {
                var match = GetCurrentMatch();
                if (match == null)
                    return false;

                return match.Template.ToString().Contains("/json");
            }
        }

        public string GetYearLink(object values)
        {
            var template = IsJsonRequest ? AutomobileServiceConstants.MakesJson : AutomobileServiceConstants.Makes;
            return RestHelpers.GetLink(template, values).ToString();
        }

        public string GetMakeLink(object values)
        {
            var template = IsJsonRequest ? AutomobileServiceConstants.ModelsJson : AutomobileServiceConstants.Models;
            return RestHelpers.GetLink(template, values).ToString();
        }

        public string GetModelLink(object values)
        {
            var template = IsJsonRequest ? AutomobileServiceConstants.EnginesJson : AutomobileServiceConstants.Engines;
            return RestHelpers.GetLink(template, values).ToString();
        }

        public string GetEngineLink(object values)
        {
            var template = IsJsonRequest ? AutomobileServiceConstants.EngineJson : AutomobileServiceConstants.Engine;
            return RestHelpers.GetLink(template, values).ToString();
        }
        #endregion

        /// <summary>
        /// This is a demo page which returns a basic html document for use during the demo only
        /// </summary>
        /// <returns>An Html Page with links to sample data.</returns>
        public Message GetDemoPage()
        {
            return RestHelpers.CreateMessageFromFile("demo.html", "text/html");
        }
    }
}