using System;
using System.Net;
using System.Web.Mvc;
using AutomobileSampleDataModel;

namespace MvcAutomobileService.Controllers
{
    public class MainController : RestController
    {
        public ActionResult Demo()
        {
            return HtmlView("demo.html");
        } 

        public ActionResult Main(int? year, string make, string model, string engine)
        {
            return MainInternal(year, make, model, engine, Request.RequestType, true);
        }

        public ActionResult JsonMain(int? year, string make, string model, string engine)
        {
            return MainInternal(year, make, model, engine, Request.RequestType, false);
        }

        private ActionResult MainInternal(int? year, string make, string model, string engine, string requestType, bool useXml)
        {
            if (requestType.Equals("GET", StringComparison.OrdinalIgnoreCase))
            {
                return HandleGet(year, make, model, engine, useXml);
            }
            else if (requestType.Equals("POST", StringComparison.OrdinalIgnoreCase))
            {
                // Insert
                return HandlePost(year, make, model, engine, useXml);
            }
            else if (requestType.Equals("PUT", StringComparison.OrdinalIgnoreCase))
            {
                // Update
                return HandlePut(year, make, model, engine, useXml);
            }
            else if (requestType.Equals("DELETE", StringComparison.OrdinalIgnoreCase))
            {
                // Delete
                return HandleDelete(year, make, model, engine);
            }
            else
            {
                return HttpStatusView(HttpStatusCode.MethodNotAllowed);
            }
        }

        public ActionResult HandleGet(int? year, string make, string model, string engine, bool useXml)
        {
            var resultsDelegate = useXml ? new Func<object, ActionResult>(Xml) : new Func<object, ActionResult>(JsonAllowGet);
            var linkBuilder = useXml ? new Func<object, string>(BuildXmlLink) : new Func<object, string>(BuildJsonLink);

            // Get
            if (!string.IsNullOrEmpty(engine))
            {
                var result = AutomobileRepository.Instance.GetAutomobile(year ?? 0, make, model, engine, linkBuilder);
                if (result == null)
                {
                    return HttpStatusView(HttpStatusCode.NotFound);
                }
                return resultsDelegate(result);
            }
            if (!string.IsNullOrEmpty(model))
            {
                return resultsDelegate(AutomobileRepository.Instance.GetEngines(year ?? 0, make, model, linkBuilder));
            }
            if (!string.IsNullOrEmpty(make))
            {
                return resultsDelegate(AutomobileRepository.Instance.GetModels(year ?? 0, make, linkBuilder));
            }
            if (year.HasValue)
            {
                return resultsDelegate(AutomobileRepository.Instance.GetMakes(year.Value, linkBuilder));
            }
            return resultsDelegate(AutomobileRepository.Instance.GetYears(linkBuilder));
        }

        public ActionResult HandlePost(int? year, string make, string model, string engine, bool useXml)
        {
            var automobile = useXml ? GetPostedXml<Automobile>() : GetPostedJson<Automobile>();
            if (automobile == null)
            {
                return HttpStatusView(HttpStatusCode.BadRequest);
            }

            var result = AutomobileRepository.Instance.Insert(automobile);
            if (result == 0)
            {
                return HttpStatusView(HttpStatusCode.NotFound);
            }
            else
            {
                Response.StatusCode = (int)HttpStatusCode.Created;
                return HandleGet(automobile.Year, automobile.Make, automobile.Model, automobile.Engine, useXml);
            }
        }

        public ActionResult HandlePut(int? year, string make, string model, string engine, bool useXml)
        {
            var automobile = useXml ? GetPostedXml<Automobile>() : GetPostedJson<Automobile>();
            if (automobile == null)
            {
                return HttpStatusView(HttpStatusCode.BadRequest);
            }

            var result = AutomobileRepository.Instance.Update(year ?? 0, make, model, engine, automobile);
            if (result == 0)
            {
                return HttpStatusView(HttpStatusCode.NotFound);
            }
            else
            {
                return HandleGet(automobile.Year, automobile.Make, automobile.Model, automobile.Engine, useXml);
            }
        }

        private ActionResult HandleDelete(int? year, string make, string model, string engine)
        {
            var result = AutomobileRepository.Instance.Delete(year ?? 0, make, model, engine);
            if (result)
            {
                return HttpStatusView(HttpStatusCode.OK);
            }
            else
            {
                return HttpStatusView(HttpStatusCode.NotFound);
            }
        }

        public string BuildXmlLink(object values)
        {
            return BuildLink("Default", values);
        }

        public string BuildJsonLink(object values)
        {
            return BuildLink("JsonDefault", values);
        }

        public string BuildLink(string routeName, object values)
        {
            var urlHelper = new UrlHelper(ControllerContext.RequestContext);

            return BaseUri + urlHelper.RouteUrl(routeName, values);
        }
    }
}