﻿using System.Net;
using System.Runtime.Serialization.Json;
using System.ServiceModel.Channels;
using System.ServiceModel.Web;

namespace WcfRestUtilities
{
    /// <summary>
    /// Contains a series of extension methods for the <see cref="System.ServiceModel.Channels.Message" /> object
    /// </summary>
    public static class MessageExtensions
    {
        /// <summary>
        /// Retrieves the body of the message using the <see cref="System.Runtime.Serialization.Json.DataContractJsonSerializer" />.
        /// </summary>
        /// <typeparam name="T">The type of object to be deserialized</typeparam>
        /// <returns>The object deserialized from the Message body.</returns>
        public static T GetJsonBody<T>(this Message message)
        {
            var serializer = new DataContractJsonSerializer(typeof(T));
            return message.GetBody<T>(serializer);
        }

        /// <summary>
        /// Retrieves the body of the message using the <see cref="System.Runtime.Serialization.XmlObjectSerializer" />.
        /// </summary>
        /// <typeparam name="T">The type of object to be deserialized</typeparam>
        /// <returns>The object deserialized from the Message body.</returns>
        public static T GetXmlBody<T>(this Message message)
        {
            var serializer = new XmlSerializerObjectSerializer(typeof(T));
            return message.GetBody<T>(serializer);
        }

        /// <summary>
        /// Adds the <see cref="System.ServiceModel.Channels.HttpResponseMessageProperty" /> with the StatusCode from the OutgoingResponse to the supplied message.  
        /// This method requires a valid <see cref="System.ServiceModel.Web.WebOperationContext" /> to work.
        /// </summary>
        /// <param name="message">The message to apply the status code to.</param>
        /// <returns>The original message with the <see cref="System.ServiceModel.Channels.HttpResponseMessageProperty" /> StatusCode set.</returns>
        public static Message WithHttpStatus(this Message message)
        {
            if (WebOperationContext.Current != null)
            {
                return WithHttpStatus(message, WebOperationContext.Current.OutgoingResponse.StatusCode);
            }
            return message;
        }

        /// <summary>
        /// Adds the <see cref="System.ServiceModel.Channels.HttpResponseMessageProperty" /> with the <see cref="System.Net.HttpStatusCode" /> to the supplied message.
        /// </summary>
        /// <param name="message">The message to apply the status code to.</param>
        /// <param name="statusCode">That stutus code to be applied to the message.</param>
        /// <returns>The original message with the <see cref="System.ServiceModel.Channels.HttpResponseMessageProperty" /> StatusCode set.</returns>
        public static Message WithHttpStatus(this Message message, HttpStatusCode statusCode)
        {
            HttpResponseMessageProperty responseProperties = null;
            if (message.Properties.ContainsKey(HttpResponseMessageProperty.Name))
            {
                responseProperties = message.Properties[HttpResponseMessageProperty.Name] as HttpResponseMessageProperty;
            }
            if (responseProperties == null)
            {
                responseProperties = new HttpResponseMessageProperty();
            }
            
            // set the StatusCode
            responseProperties.StatusCode = statusCode;

            //set response properties
            message.Properties[HttpResponseMessageProperty.Name] = responseProperties;

            return message;
        }
    }
}
