﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization.Json;
using System.ServiceModel.Channels;
using System.ServiceModel.Web;
using System.Xml;

namespace WcfRestUtilities
{
    /// <summary>
    /// Containes a series of static helper methods for use when serving RESTful Services in WCF
    /// </summary>
    public class RestHelpers
    {
        /// <summary>
        /// Attempts to populate properties on a new instance of object T with values matched from 
        /// the <see cref="System.UriTemplateMatch" />.BoundVariables collection
        /// </summary>
        /// <typeparam name="T">The type to instantiate and populate</typeparam>
        /// <param name="match">the <see cref="System.UriTemplateMatch" /> whose BoundVariables will be used to populate a new instance of object T</param>
        /// <returns>A fully populated instance of object T.  If match is null and empty T object will be returned.</returns>
        public static T GetMatchedValues<T>(UriTemplateMatch match) where T : new()
        {
            var matchValues = new T();
            if (match == null)
                return matchValues;

            var properties = TypeDescriptor.GetProperties(matchValues).Cast<PropertyDescriptor>();
            foreach (string k in match.BoundVariables.Keys)
            {
                string key = k;
                var p = properties.FirstOrDefault(pi => pi.Name.Equals(key, StringComparison.OrdinalIgnoreCase));
                if (p != null && !p.IsReadOnly)
                {
                    p.SetValue(matchValues, match.BoundVariables[k]);
                }
            }
            return matchValues;
        }

        /// <summary>
        /// Uses the <see cref="System.ServiceModel.Web.WebOperationContext" />'s IncomingRequest to in conjunction with the templateTable supplied to find
        /// a match based on the requested Uri.
        /// </summary>
        /// <param name="templateTable">The table containing possible UriTemplateMatches</param>
        /// <returns>A single instance with the first UriTemplateMatch, otherwise null.</returns>
        public static UriTemplateMatch GetCurrentMatch(UriTemplateTable templateTable)
        {
            var request = WebOperationContext.Current.IncomingRequest;
            var matches = templateTable.Match(request.UriTemplateMatch.RequestUri);
            return matches.FirstOrDefault();
        }

        #region Get Link
        /// <summary>
        /// Uses an anonymous object values to generate a link by Binding the object properties by name to the 
        /// templates variables
        /// </summary>
        /// <param name="template">The template to bind values by name to.</param>
        /// <param name="values">An anonymous object whose properties will be bound.</param>
        /// <returns>A uri for use when linking to a template</returns>
        public static Uri GetLink(UriTemplate template, object values)
        {
            return GetLink(template, new TemplateValueDictionary(values));
        }

        /// <summary>
        /// Uses a dictionary of values to generate a link by Binding the values by key to the 
        /// templates variables
        /// </summary>
        /// <param name="template">The template to bind values by name to.</param>
        /// <param name="values">An dictionary of values that will be bound.</param>
        /// <returns>A uri for use when linking to a template</returns>
        public static Uri GetLink(UriTemplate template, IDictionary<string, string> values)
        {
            var currentUri = WebOperationContext.Current.IncomingRequest.UriTemplateMatch.BaseUri;
            return template.BindByName(currentUri, values);
        } 
        #endregion

        #region Create Messages
        /// <summary>
        /// Creates a <see cref="System.ServiceModel.Channels.Message" /> object using the <see cref="System.Runtime.Serialization.Json.DataContractJsonSerializer" /> for the supplied type T.
        /// </summary>
        /// <typeparam name="T">The type that the <see cref="System.Runtime.Serialization.Json.DataContractJsonSerializer" /> will be based on.</typeparam>
        /// <param name="result">The object to be serialized into a message</param>
        /// <returns>A Message for use in WCF communication</returns>
        public static Message CreateJsonMessage<T>(T result)
        {
            var serializer = new DataContractJsonSerializer(typeof(T));
            var message = Message.CreateMessage(MessageVersion.None, string.Empty, result, serializer);
            message.Properties.Add(WebBodyFormatMessageProperty.Name, new WebBodyFormatMessageProperty(WebContentFormat.Json));
            return message;
        }

        /// <summary>
        /// Creates a <see cref="System.ServiceModel.Channels.Message" /> object using the <see cref="System.Xml.Serialization.XmlSerializer" /> for the supplied type T.
        /// </summary>
        /// <typeparam name="T">The type that the <see cref="System.Xml.Serialization.XmlSerializer" /> will be based on.</typeparam>
        /// <param name="result">The object to be serialized into a message</param>
        /// <returns>A Message for use in WCF communication</returns>
        public static Message CreateXmlSerializedMessage<T>(T result)
        {
            var serializer = new XmlSerializerObjectSerializer(typeof(T));
            return Message.CreateMessage(MessageVersion.None, string.Empty, result, serializer);
        }

        /// <summary>
        /// Creates an empty <see cref="System.ServiceModel.Channels.Message" /> object.
        /// </summary>
        /// <returns>A Message for use in WCF communication</returns>
        public static Message CreateEmptyMessage()
        {
            return Message.CreateMessage(MessageVersion.None, string.Empty, EmptyBodyWriter.Instance);
        }

        /// <summary>
        /// Creates a <see cref="System.ServiceModel.Channels.Message" /> object based on a File.  This method is commonly used to serve static files such as Html files.
        /// </summary>
        /// <param name="filename">The filename to the file that will be used to generate the <see cref="System.ServiceModel.Channels.Message" />.</param>
        /// <returns>A Message for use in WCF communication</returns>
        public static Message CreateMessageFromFile(string filename)
        {
            return CreateMessageFromFile(filename, null);
        }

        /// <summary>
        /// Creates a <see cref="System.ServiceModel.Channels.Message" /> object based on a File.  This method is commonly used to serve static files such as Html files.
        /// </summary>
        /// <param name="filename">The filename to the file that will be used to generate the <see cref="System.ServiceModel.Channels.Message" />.</param>
        /// <param name="contentType">The contentType to be set on the OutgoingResponse. This requires a valid <see cref="System.ServiceModel.Web.WebOperationContext" /> to be set.</param>
        /// <returns>A Message for use in WCF communication</returns>
        public static Message CreateMessageFromFile(string filename, string contentType)
        {
            if (WebOperationContext.Current != null && !string.IsNullOrEmpty(contentType))
            {
                WebOperationContext.Current.OutgoingResponse.ContentType = contentType;
            }
            var xdr = XmlDictionaryReader.CreateTextReader((Stream)File.Open(filename, FileMode.Open), XmlDictionaryReaderQuotas.Max);
            return Message.CreateMessage(MessageVersion.None, string.Empty, xdr);
        } 
        #endregion

        #region Set Status Codes
        /// <summary>
        /// Sets the StatusCode of the OutgoingResponse to the supplied StatusCode.  Requires a valid <see cref="System.ServiceModel.Web.WebOperationContext" />.
        /// </summary>
        /// <param name="statusCode">The HttpStatusCode to the set.</param>
        public static void SetStatus(HttpStatusCode statusCode)
        {
            if (WebOperationContext.Current != null)
            {
                WebOperationContext.Current.OutgoingResponse.StatusCode = statusCode;
            }
        }

        /// <summary>
        /// Sets the status of the OutgoingResponse to 200 (OK).
        /// </summary>
        public static void SetStatus200()
        {
            SetStatus(HttpStatusCode.OK);
        }

        /// <summary>
        /// Sets the status of the OutgoingResponse to 200 (OK).
        /// </summary>
        public static void SetStatusOK()
        {
            SetStatus(HttpStatusCode.OK);
        }

        /// <summary>
        /// Sets the status of the OutgoingResponse to 201 (Created).
        /// </summary>
        public static void SetStatus201()
        {
            SetStatus(HttpStatusCode.Created);
        }

        /// <summary>
        /// Sets the status of the OutgoingResponse to 201 (Created).
        /// </summary>
        public static void SetStatusCreated()
        {
            SetStatus(HttpStatusCode.Created);
        }

        /// <summary>
        /// Sets the status of the OutgoingResponse to 400 (Bad Request).
        /// </summary>
        public static void SetStatus400()
        {
            SetStatus(HttpStatusCode.BadRequest);
        }

        /// <summary>
        /// Sets the status of the OutgoingResponse to 400 (Bad Request).
        /// </summary>
        public static void SetStatusBadRequest()
        {
            SetStatus(HttpStatusCode.BadRequest);
        }

        /// <summary>
        /// Sets the status of the OutgoingResponse to 404 (Not Found).
        /// </summary>
        public static void SetStatus404()
        {
            SetStatus(HttpStatusCode.NotFound);
        }

        /// <summary>
        /// Sets the status of the OutgoingResponse to 404 (Not Found).
        /// </summary>
        public static void SetStatusNotFound()
        {
            SetStatus(HttpStatusCode.NotFound);
        }

        /// <summary>
        /// Sets the status of the OutgoingResponse to 405 (MethodNotAllowed).
        /// </summary>
        public static void SetStatus405()
        {
            SetStatus(HttpStatusCode.MethodNotAllowed);
        }

        /// <summary>
        /// Sets the status of the OutgoingResponse to 405 (MethodNotAllowed).
        /// </summary>
        public static void SetStatusMethodNotAllowed()
        {
            SetStatus(HttpStatusCode.MethodNotAllowed);
        } 
        #endregion
    }
}
