using System;
using System.ServiceModel;
using System.ServiceModel.Web;
using WcfRestUtilities.Logger;

namespace WcfRestUtilities
{
    /// <summary>
    /// Creates an instance of a 'Safe' <see cref="System.ServiceModel.Web.WebServiceHost" />.  This class is responsible for safely Opening and Closing a potentially Faulted endpoint.
    /// </summary>
    public class SafeWebServiceHost
    {
        /// <summary>
        /// The inner logger used to notify the system of the status
        /// </summary>
        private readonly ILogger _logger;
        /// <summary>
        /// The inner implementation of the <see cref="System.ServiceModel.Web.WebServiceHost" />
        /// </summary>
        private readonly WebServiceHost _serviceHost;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of SafeWebServiceHost.
        /// </summary>
        /// <param name="serviceType">The service type to be hosted</param>
        /// <param name="baseAddress">The base address to host the service</param>
        public SafeWebServiceHost(Type serviceType, string baseAddress)
            : this(serviceType, baseAddress, new NullLogger(), null)
        {
        }

        /// <summary>
        /// Initializes a new instance of SafeWebServiceHost.
        /// </summary>
        /// <param name="serviceType">The service type to be hosted</param>
        /// <param name="baseAddress">The base address to host the service</param>
        /// <param name="logger">The logger to be used for logging status messages</param>
        public SafeWebServiceHost(Type serviceType, string baseAddress, ILogger logger)
            : this(serviceType, baseAddress, logger, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of SafeWebServiceHost.
        /// </summary>
        /// <param name="serviceType">The service type to be hosted</param>
        /// <param name="baseAddress">The base address to host the service</param>
        /// <param name="serviceName">The service name to be used when logging.  If null, the name will be generated from the serviceType Name.</param>
        public SafeWebServiceHost(Type serviceType, string baseAddress, string serviceName)
            : this(serviceType, baseAddress, new NullLogger(), serviceName)
        {
        }

        /// <summary>
        /// Creates a new instance of the SafeWebServiceHost.
        /// </summary>
        /// <param name="serviceType">The Service Type to be hosted</param>
        /// <param name="baseAddress">The base web address to use.</param>
        /// <param name="logger">An optional Logger.  Commonly a ConsoleLogger is used, but by default the NullLogger will be used.</param>
        /// <param name="serviceName">The service name to be used when logging.  If null, the name will be generated from the serviceType Name.</param>
        public SafeWebServiceHost(Type serviceType, string baseAddress, ILogger logger, string serviceName)
        {
            _logger = logger;
            BaseAddress = baseAddress;
            ServiceName = serviceName;
            if (string.IsNullOrEmpty(serviceName))
            {
                ServiceName = serviceType.Name;
            }

            _serviceHost = new WebServiceHost(serviceType, new Uri(baseAddress));
        } 
        #endregion

        /// <summary>
        /// The base address that will be used to host the service.
        /// </summary>
        public string BaseAddress { get; set; }
        /// <summary>
        /// The nme of the service to be used when logging.
        /// </summary>
        public string ServiceName { get; set; }

        /// <summary>
        /// Safely opens and instance of the Service Type.
        /// </summary>
        /// <returns>True if successful, otherwise false.</returns>
        public bool SafeOpen()
        {
            bool openSucceeded = false;

            var address = BaseAddress ?? Properties.Resources.UnknownServiceName;
            try
            {
                address = _serviceHost.Description.Endpoints[0].Address.Uri.ToString();
            }
            catch { }

            try
            {
                _serviceHost.Open();

                openSucceeded = true;
            }
            catch (AddressAccessDeniedException ex)
            {
                _logger.WriteError(Properties.Resources.AddressAccessDeniedMessage, ex);
            }
            catch (Exception ex)
            {
                _logger.WriteError(ex);
            }
            finally
            {
                if (!openSucceeded)
                {
                    _serviceHost.Abort();
                }
            }

            if (_serviceHost.State == CommunicationState.Opened)
            {
                _logger.WriteLine(string.Format(Properties.Resources.ServiceIsRunning, ServiceName, address));
            }
            else
            {
                _logger.WriteLine(string.Format(Properties.Resources.ServiceFailedToOpen, ServiceName));
            }

            return openSucceeded;
        }

        /// <summary>
        /// Safely closes and instance of the Service Type.
        /// </summary>
        /// <returns>True if successful, otherwise false.</returns>
        public bool SafeClose()
        {
            bool closeSucceeded = false;

            try
            {
                _serviceHost.Close();

                closeSucceeded = true;

                _logger.WriteLine(string.Format(Properties.Resources.ServiceHasClosed, ServiceName));
            }
            catch (Exception ex)
            {
                _logger.WriteError(string.Format(Properties.Resources.ServiceFailedToClose, ServiceName), ex);
            }
            finally
            {
                if (!closeSucceeded)
                {
                    _serviceHost.Abort();
                }
            }

            return closeSucceeded;
        }
    }
}